@@ -6,6 +6,8 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include "config-kactivities.h"
+
 #include "mainwindow.h"
 #include "krdc_debug.h"
 #include "settings.h"
@@ -31,6 +33,10 @@
 #include <KToggleFullScreenAction>
 #include <KToolBar>
 
+#if HAVE_KACTIVITIES
+#include <KActivities/ResourceInstance>
+#endif
+
 #include <QClipboard>
 #include <QDockWidget>
 #include <QFontMetrics>
@@ -289,7 +295,6 @@ void MainWindow::newConnection(const QUrl &newUrl, bool switchFullscreenWhenConn
     connect(view, SIGNAL(statusChanged(RemoteView::RemoteStatus)), this, SLOT(statusChanged(RemoteView::RemoteStatus)));
     connect(view, SIGNAL(disconnected()), this, SLOT(disconnectHost()));
 
-    view->winId();  // native widget workaround for bug 253365
     QScrollArea *scrollArea = createScrollArea(m_tabWidget, view);
 
     const int indexOfNewConnectionWidget = m_tabWidget->indexOf(m_newConnectionWidget);
@@ -304,6 +309,10 @@ void MainWindow::newConnection(const QUrl &newUrl, bool switchFullscreenWhenConn
     view->start();
     setFactor(view->hostPreferences()->scaleFactor());
 
+#if HAVE_KACTIVITIES
+    KActivities::ResourceInstance::notifyAccessed(url, QGuiApplication::desktopFileName());
+#endif
+
     Q_EMIT factorUpdated(view->hostPreferences()->scaleFactor());
     Q_EMIT scaleUpdated(scale_state);
 }
@@ -502,7 +511,6 @@ void MainWindow::switchFullscreen()
         KToggleFullScreenAction::setFullScreen(m_fullscreenWindow, true);
 
         MinimizePixel *minimizePixel = new MinimizePixel(m_fullscreenWindow);
-        minimizePixel->winId(); // force it to be a native widget (prevents problem with QX11EmbedContainer)
         connect(minimizePixel, SIGNAL(rightClicked()), m_fullscreenWindow, SLOT(showMinimized()));
         m_fullscreenWindow->installEventFilter(this);
 
@@ -588,8 +596,7 @@ void MainWindow::closeTab(int index)
     bool isNewConnectionPage = widget == m_newConnectionWidget;
 
     if (!isNewConnectionPage) {
-        RemoteView *view = m_remoteViewMap.value(widget);
-        m_remoteViewMap.remove(m_remoteViewMap.key(view));
+        RemoteView *view = m_remoteViewMap.take(widget);
         view->startQuitting();
         widget->deleteLater();
     }
@@ -802,7 +809,6 @@ void MainWindow::showRemoteViewToolbar()
 
     if (!m_toolBar) {
         m_toolBar = new FloatingToolBar(m_fullscreenWindow, m_fullscreenWindow);
-        m_toolBar->winId(); // force it to be a native widget (prevents problem with QX11EmbedContainer)
         m_toolBar->setSide(FloatingToolBar::Top);
 
         KComboBox *sessionComboBox = new KComboBox(m_toolBar);
